"use strict";

/* exported Promise*/

var AbstractAdminPanel = require('./AbstractAdminPanel');

var Functions = require('../../Functions');
var GuiConnector = require('../../GuiConnector');

/* exported logger */
// noinspection JSUnusedLocalSymbols
var logger = require('../../logger');

// noinspection JSUnusedLocalSymbols
var Promise = require("bluebird");

function ConfigurationAdminPanel(params) {
  AbstractAdminPanel.call(this, params);

  var self = this;

  self._createGui();
}

ConfigurationAdminPanel.prototype = Object.create(AbstractAdminPanel.prototype);
ConfigurationAdminPanel.prototype.constructor = ConfigurationAdminPanel;

ConfigurationAdminPanel.prototype._createGui = function () {
  var self = this;
  var configurationDiv = Functions.createElement({
    type: "div"
  });
  self.getElement().appendChild(configurationDiv);

  var configurationTable = Functions.createElement({
    type: "table",
    name: "configurationTable",
    className: "display",
    style: "width:100%"
  });
  configurationDiv.appendChild(configurationTable);

  // noinspection JSUnusedGlobalSymbols
  $(configurationTable).DataTable({
    fnRowCallback: function (nRow, aData) {
      nRow.setAttribute('id', aData[0]);
    },
    columns: [{
      title: 'Name'
    }, {
      title: 'Value'
    }, {
      title: 'Save'
    }]
  });
  $(configurationTable).on("click", "[name='saveOption']", function () {
    var button = this;
    return self.saveOption($(button).attr("data")).then(null, GuiConnector.alert);
  });
  $(configurationTable).on("input", ".minerva-color-input", function () {
    var input = this;
    var value = $(input).val();
    var type = $(input).attr("data");
    if (value.length !== 6) {
      value = "FFFFFF";
    }
    $("[name='edit-color-" + type + "']", configurationTable).css("background-color", "#" + value);
  });
  $(configurationTable).on("click", ".minerva-color-button", function () {
    var button = this;
    var value = $(button).css("background-color");
    var type = $(button).attr("data");
    logger.debug(type);
    var colorPicker = $(button).parent().spectrum({
      color: value,
      move: function (color) {
        var value = color.toHexString().replace("#", '');
        $("[name='edit-" + type + "']", configurationTable).val(value);
        $("[name='edit-color-" + type + "']", configurationTable).css("background-color", "#" + value);
      },
      hide: function () {
        colorPicker.spectrum("destroy");
      }
    });
    return new Promise.delay(1).then(function () {
      colorPicker.show();
      colorPicker.spectrum("show");
    });
  });
};

ConfigurationAdminPanel.prototype.init = function () {
  var self = this;
  return ServerConnector.getConfiguration().then(function (configuration) {
    self.setOptions(configuration.getOptions());
  });
};

ConfigurationAdminPanel.prototype.setOptions = function (options) {
  var self = this;
  var dataTable = $($("[name='configurationTable']", self.getElement())[0]).DataTable();
  var data = [];
  for (var i = 0; i < options.length; i++) {
    var option = options[i];
    var rowData = self.optionToTableRow(option);
    data.push(rowData);
  }
  dataTable.clear().rows.add(data).draw();
};

ConfigurationAdminPanel.prototype.optionToTableRow = function (option) {
  var value = option.getValue();
  var row = [];
  var editOption;
  if (option.getValueType() === "STRING" ||
    option.getValueType() === "INTEGER" ||
    option.getValueType() === "DOUBLE" ||
    option.getValueType() === "EMAIL" ||
    option.getValueType() === "PASSWORD" ||
    option.getValueType() === "URL") {
    editOption = "<input name='edit-" + option.getType() + "' value='" + value + "'/>";
  } else if (option.getValueType() === "COLOR") {
    editOption = "<div>" +
      "<input class='minerva-color-input' name='edit-" + option.getType() + "' data='" + option.getType() + "' value='" + value + "'/>" +
      "<button class='minerva-color-button' name='edit-color-" + option.getType() + "' data='" + option.getType() + "' style='background-color: #" + value + "'>&nbsp</button>" +
      "</div>";
  } else {
    logger.warn("Don't know how to handle: " + option.getValueType());
    editOption = "<input name='edit-" + option.getType() + "' value='" + value + "'readonly/>";
  }
  row[0] = option.getCommonName();
  row[1] = editOption;
  row[2] = "<button name='saveOption' data='" + option.getType() + "'>SAVE</button>";
  return row;
};

ConfigurationAdminPanel.prototype.saveOption = function (type) {
  var self = this;
  return ServerConnector.getConfiguration().then(function (configuration) {
    var option = configuration.getOption(type);
    var value = $("[name='edit-" + type + "']", self.getElement()).val();
    option.setValue(value);
    return ServerConnector.updateConfigurationOption(option);
  });
};
ConfigurationAdminPanel.prototype.destroy = function () {
  var self = this;
  var table = $("[name='configurationTable']", self.getElement())[0];
  if ($.fn.DataTable.isDataTable(table)) {
    $(table).DataTable().destroy();
  }

};

module.exports = ConfigurationAdminPanel;
